@@ -1,5 +1,3 @@
-#define GL_GLEXT_PROTOTYPES 1
-
 #include <epoxy/gl.h>
 #include <assert.h>
 #include <math.h>
@@ -37,6 +35,8 @@ namespace movit {
 EffectChain::EffectChain(float aspect_nom, float aspect_denom, ResourcePool *resource_pool)
 	: aspect_nom(aspect_nom),
 	  aspect_denom(aspect_denom),
+	  output_color_rgba(false),
+	  output_color_ycbcr(false),
 	  dither_effect(NULL),
 	  num_dither_bits(0),
 	  output_origin(OUTPUT_ORIGIN_BOTTOM_LEFT),
@@ -49,6 +49,14 @@ EffectChain::EffectChain(float aspect_nom, float aspect_denom, ResourcePool *res
 	} else {
 		owns_resource_pool = false;
 	}
+
+	// Generate a VBO with some data in (shared position and texture coordinate data).
+	float vertices[] = {
+		0.0f, 2.0f,
+		0.0f, 0.0f,
+		2.0f, 0.0f
+	};
+	vbo = generate_vbo(2, GL_FLOAT, sizeof(vertices), vertices);
 }
 
 EffectChain::~EffectChain()
@@ -64,6 +72,8 @@ EffectChain::~EffectChain()
 	if (owns_resource_pool) {
 		delete resource_pool;
 	}
+	glDeleteBuffers(1, &vbo);
+	check_error();
 }
 
 Input *EffectChain::add_input(Input *input)
@@ -77,18 +87,20 @@ Input *EffectChain::add_input(Input *input)
 void EffectChain::add_output(const ImageFormat &format, OutputAlphaFormat alpha_format)
 {
 	assert(!finalized);
+	assert(!output_color_rgba);
 	output_format = format;
 	output_alpha_format = alpha_format;
-	output_color_type = OUTPUT_COLOR_RGB;
+	output_color_rgba = true;
 }
 
 void EffectChain::add_ycbcr_output(const ImageFormat &format, OutputAlphaFormat alpha_format,
                                    const YCbCrFormat &ycbcr_format, YCbCrOutputSplitting output_splitting)
 {
 	assert(!finalized);
+	assert(!output_color_ycbcr);
 	output_format = format;
 	output_alpha_format = alpha_format;
-	output_color_type = OUTPUT_COLOR_YCBCR;
+	output_color_ycbcr = true;
 	output_ycbcr_format = ycbcr_format;
 	output_ycbcr_splitting = output_splitting;
 
@@ -180,6 +192,13 @@ GLenum EffectChain::get_input_sampler(Node *node, unsigned input_num) const
 	return GL_TEXTURE0 + node->incoming_links[input_num]->bound_sampler_num;
 }
 
+GLenum EffectChain::has_input_sampler(Node *node, unsigned input_num) const
+{
+	assert(input_num < node->incoming_links.size());
+	return node->incoming_links[input_num]->bound_sampler_num >= 0 &&
+		node->incoming_links[input_num]->bound_sampler_num < 8;
+}
+
 void EffectChain::find_all_nonlinear_inputs(Node *node, vector<Node *> *nonlinear_inputs)
 {
 	if (node->output_gamma_curve == GAMMA_LINEAR &&
@@ -208,7 +227,7 @@ Effect *EffectChain::add_effect(Effect *effect, const vector<Effect *> &inputs)
 	return effect;
 }
 
-// GLSL pre-1.30 doesn't support token pasting. Replace PREFIX(x) with <effect_id>_x.
+// ESSL doesn't support token pasting. Replace PREFIX(x) with <effect_id>_x.
 string replace_prefix(const string &text, const string &prefix)
 {
 	string output;
@@ -365,22 +384,37 @@ void EffectChain::compile_glsl_program(Phase *phase)
 	frag_shader += string("#define INPUT ") + phase->effect_ids[phase->effects.back()] + "\n";
 
 	// If we're the last phase, add the right #defines for Y'CbCr multi-output as needed.
-	if (phase->output_node->outgoing_links.empty() && output_color_type == OUTPUT_COLOR_YCBCR) {
+	vector<string> frag_shader_outputs;  // In order.
+	if (phase->output_node->outgoing_links.empty() && output_color_ycbcr) {
 		switch (output_ycbcr_splitting) {
 		case YCBCR_OUTPUT_INTERLEAVED:
 			// No #defines set.
+			frag_shader_outputs.push_back("FragColor");
 			break;
 		case YCBCR_OUTPUT_SPLIT_Y_AND_CBCR:
 			frag_shader += "#define YCBCR_OUTPUT_SPLIT_Y_AND_CBCR 1\n";
+			frag_shader_outputs.push_back("Y");
+			frag_shader_outputs.push_back("Chroma");
 			break;
 		case YCBCR_OUTPUT_PLANAR:
 			frag_shader += "#define YCBCR_OUTPUT_PLANAR 1\n";
+			frag_shader_outputs.push_back("Y");
+			frag_shader_outputs.push_back("Cb");
+			frag_shader_outputs.push_back("Cr");
 			break;
 		default:
 			assert(false);
 		}
+
+		if (output_color_rgba) {
+			// Note: Needs to come in the header, because not only the
+			// output needs to see it (YCbCrConversionEffect and DitherEffect
+			// do, too).
+			frag_shader_header += "#define YCBCR_ALSO_OUTPUT_RGBA 1\n";
+			frag_shader_outputs.push_back("RGBA");
+		}
 	}
-	frag_shader.append(read_version_dependent_file("footer", "frag"));
+	frag_shader.append(read_file("footer.frag"));
 
 	// Collect uniforms from all effects and output them. Note that this needs
 	// to happen after output_fragment_shader(), even though the uniforms come
@@ -400,7 +434,9 @@ void EffectChain::compile_glsl_program(Phase *phase)
 		extract_uniform_declarations(effect->uniforms_vec2, "vec2", effect_id, &phase->uniforms_vec2, &frag_shader_uniforms);
 		extract_uniform_declarations(effect->uniforms_vec3, "vec3", effect_id, &phase->uniforms_vec3, &frag_shader_uniforms);
 		extract_uniform_declarations(effect->uniforms_vec4, "vec4", effect_id, &phase->uniforms_vec4, &frag_shader_uniforms);
+		extract_uniform_array_declarations(effect->uniforms_float_array, "float", effect_id, &phase->uniforms_float, &frag_shader_uniforms);
 		extract_uniform_array_declarations(effect->uniforms_vec2_array, "vec2", effect_id, &phase->uniforms_vec2, &frag_shader_uniforms);
+		extract_uniform_array_declarations(effect->uniforms_vec3_array, "vec3", effect_id, &phase->uniforms_vec3, &frag_shader_uniforms);
 		extract_uniform_array_declarations(effect->uniforms_vec4_array, "vec4", effect_id, &phase->uniforms_vec4, &frag_shader_uniforms);
 		extract_uniform_declarations(effect->uniforms_mat3, "mat3", effect_id, &phase->uniforms_mat3, &frag_shader_uniforms);
 	}
@@ -419,7 +455,15 @@ void EffectChain::compile_glsl_program(Phase *phase)
 		vert_shader[pos + needle.size() - 1] = '1';
 	}
 
-	phase->glsl_program_num = resource_pool->compile_glsl_program(vert_shader, frag_shader);
+	phase->glsl_program_num = resource_pool->compile_glsl_program(vert_shader, frag_shader, frag_shader_outputs);
+	GLint position_attribute_index = glGetAttribLocation(phase->glsl_program_num, "position");
+	GLint texcoord_attribute_index = glGetAttribLocation(phase->glsl_program_num, "texcoord");
+	if (position_attribute_index != -1) {
+		phase->attribute_indexes.insert(position_attribute_index);
+	}
+	if (texcoord_attribute_index != -1) {
+		phase->attribute_indexes.insert(texcoord_attribute_index);
+	}
 
 	// Collect the resulting location numbers for each uniform.
 	collect_uniform_locations(phase->glsl_program_num, &phase->uniforms_sampler2d);
@@ -489,7 +533,8 @@ Phase *EffectChain::construct_phase(Node *output, map<Node *, Phase *> *complete
 			bool start_new_phase = false;
 
 			if (node->effect->needs_texture_bounce() &&
-			    !deps[i]->effect->is_single_texture()) {
+			    !deps[i]->effect->is_single_texture() &&
+			    !deps[i]->effect->override_disable_bounce()) {
 				start_new_phase = true;
 			}
 
@@ -560,9 +605,17 @@ Phase *EffectChain::construct_phase(Node *output, map<Node *, Phase *> *complete
 	// and create a GLSL program for it.
 	assert(!phase->effects.empty());
 
-	// Deduplicate the inputs.
-	sort(phase->inputs.begin(), phase->inputs.end());
-	phase->inputs.erase(unique(phase->inputs.begin(), phase->inputs.end()), phase->inputs.end());
+	// Deduplicate the inputs, but don't change the ordering e.g. by sorting;
+	// that would be nondeterministic and thus reduce cacheability.
+	// TODO: Make this even more deterministic.
+	vector<Phase *> dedup_inputs;
+	set<Phase *> seen_inputs;
+	for (size_t i = 0; i < phase->inputs.size(); ++i) {
+		if (seen_inputs.insert(phase->inputs[i]).second) {
+			dedup_inputs.push_back(phase->inputs[i]);
+		}
+	}
+	swap(phase->inputs, dedup_inputs);
 
 	// Allocate samplers for each input.
 	phase->input_samplers.resize(phase->inputs.size());
@@ -1519,8 +1572,8 @@ void EffectChain::fix_output_gamma()
 // gamma-encoded data.
 void EffectChain::add_ycbcr_conversion_if_needed()
 {
-	assert(output_color_type == OUTPUT_COLOR_RGB || output_color_type == OUTPUT_COLOR_YCBCR);
-	if (output_color_type != OUTPUT_COLOR_YCBCR) {
+	assert(output_color_rgba || output_color_ycbcr);
+	if (!output_color_ycbcr) {
 		return;
 	}
 	Node *output = find_output_node();
@@ -1635,6 +1688,12 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 {
 	assert(finalized);
 
+	// This needs to be set anew, in case we are coming from a different context
+	// from when we initialized.
+	check_error();
+	glDisable(GL_DITHER);
+	check_error();
+
 	// Save original viewport.
 	GLuint x = 0, y = 0;
 
@@ -1648,6 +1707,7 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 	}
 
 	// Basic state.
+	check_error();
 	glDisable(GL_BLEND);
 	check_error();
 	glDisable(GL_DEPTH_TEST);
@@ -1655,22 +1715,16 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 	glDepthMask(GL_FALSE);
 	check_error();
 
-	// Generate a VAO. All the phases should have exactly the same vertex attributes,
-	// so it's safe to reuse this.
-	float vertices[] = {
-		0.0f, 2.0f,
-		0.0f, 0.0f,
-		2.0f, 0.0f
-	};
-
+	// Generate a VAO that will be used during the entire execution,
+	// and bind the VBO, since it contains all the data.
 	GLuint vao;
 	glGenVertexArrays(1, &vao);
 	check_error();
 	glBindVertexArray(vao);
 	check_error();
-
-	GLuint position_vbo = fill_vertex_attribute(phases[0]->glsl_program_num, "position", 2, GL_FLOAT, sizeof(vertices), vertices);
-	GLuint texcoord_vbo = fill_vertex_attribute(phases[0]->glsl_program_num, "texcoord", 2, GL_FLOAT, sizeof(vertices), vertices);  // Same as vertices.
+	glBindBuffer(GL_ARRAY_BUFFER, vbo);
+	check_error();
+	set<GLint> bound_attribute_indices;
 
 	set<Phase *> generated_mipmaps;
 
@@ -1696,7 +1750,7 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 				CHECK(dither_effect->set_int("output_height", height));
 			}
 		}
-		execute_phase(phase, phase_num == phases.size() - 1, &output_textures, &generated_mipmaps);
+		execute_phase(phase, phase_num == phases.size() - 1, &bound_attribute_indices, &output_textures, &generated_mipmaps);
 		if (do_phase_timing) {
 			glEndQuery(GL_TIME_ELAPSED);
 		}
@@ -1713,9 +1767,10 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 	glUseProgram(0);
 	check_error();
 
-	cleanup_vertex_attribute(phases[0]->glsl_program_num, "position", position_vbo);
-	cleanup_vertex_attribute(phases[0]->glsl_program_num, "texcoord", texcoord_vbo);
-
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	check_error();
+	glBindVertexArray(0);
+	check_error();
 	glDeleteVertexArrays(1, &vao);
 	check_error();
 
@@ -1771,7 +1826,10 @@ void EffectChain::print_phase_timing()
 	printf("Total:   %5.1f ms\n", total_time_ms);
 }
 
-void EffectChain::execute_phase(Phase *phase, bool last_phase, map<Phase *, GLuint> *output_textures, set<Phase *> *generated_mipmaps)
+void EffectChain::execute_phase(Phase *phase, bool last_phase,
+                                set<GLint> *bound_attribute_indices,
+                                map<Phase *, GLuint> *output_textures,
+                                set<Phase *> *generated_mipmaps)
 {
 	GLuint fbo = 0;
 
@@ -1784,9 +1842,7 @@ void EffectChain::execute_phase(Phase *phase, bool last_phase, map<Phase *, GLui
 		output_textures->insert(make_pair(phase, tex_num));
 	}
 
-	const GLuint glsl_program_num = phase->glsl_program_num;
-	check_error();
-	glUseProgram(glsl_program_num);
+	glUseProgram(phase->glsl_program_num);
 	check_error();
 
 	// Set up RTT inputs for this phase.
@@ -1817,7 +1873,7 @@ void EffectChain::execute_phase(Phase *phase, bool last_phase, map<Phase *, GLui
 	for (unsigned i = 0; i < phase->effects.size(); ++i) {
 		Node *node = phase->effects[i];
 		unsigned old_sampler_num = sampler_num;
-		node->effect->set_gl_state(glsl_program_num, phase->effect_ids[node], &sampler_num);
+		node->effect->set_gl_state(phase->glsl_program_num, phase->effect_ids[node], &sampler_num);
 		check_error();
 
 		if (node->effect->is_single_texture()) {
@@ -1832,12 +1888,34 @@ void EffectChain::execute_phase(Phase *phase, bool last_phase, map<Phase *, GLui
 	// from there.
 	setup_uniforms(phase);
 
+	// Clean up old attributes if they are no longer needed.
+	for (set<GLint>::iterator attr_it = bound_attribute_indices->begin();
+	     attr_it != bound_attribute_indices->end(); ) {
+		if (phase->attribute_indexes.count(*attr_it) == 0) {
+			glDisableVertexAttribArray(*attr_it);
+			check_error();
+			bound_attribute_indices->erase(attr_it++);
+		} else {
+			++attr_it;
+		}
+	}
+
+	// Set up the new attributes, if needed.
+	for (set<GLint>::iterator attr_it = phase->attribute_indexes.begin();
+	     attr_it != phase->attribute_indexes.end();
+	     ++attr_it) {
+		if (bound_attribute_indices->count(*attr_it) == 0) {
+			glEnableVertexAttribArray(*attr_it);
+			check_error();
+			glVertexAttribPointer(*attr_it, 2, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(0));
+			check_error();
+			bound_attribute_indices->insert(*attr_it);
+		}
+	}
+
 	glDrawArrays(GL_TRIANGLES, 0, 3);
 	check_error();
-
-	glUseProgram(0);
-	check_error();
-
+	
 	for (unsigned i = 0; i < phase->effects.size(); ++i) {
 		Node *node = phase->effects[i];
 		node->effect->clear_gl_state();
