@@ -29,6 +29,7 @@ public:
 	virtual void set_gl_state(GLuint glsl_program_num, const std::string &prefix, unsigned *sampler_num);
 
 	virtual bool needs_srgb_primaries() const { return false; }
+	virtual bool one_to_one_sampling() const { return true; }
 
 	// Actually needs postmultiplied input as well as outputting it.
 	// EffectChain will take care of that.
@@ -36,6 +37,7 @@ public:
 
 private:
 	GammaCurve destination_curve;
+	float uniform_linear_scale, uniform_c0, uniform_c1, uniform_c2, uniform_c3, uniform_c4, uniform_beta;
 };
 
 }  // namespace movit
