@@ -30,6 +30,7 @@ public:
 
 	virtual bool needs_linear_light() const { return false; }
 	virtual bool needs_srgb_primaries() const { return false; }
+	virtual bool one_to_one_sampling() const { return true; }
 
 	// Actually processes its input in a nonlinear fashion,
 	// but does not touch alpha, and we are a special case anyway.
@@ -37,6 +38,7 @@ public:
 
 private:
 	GammaCurve source_curve;
+	float uniform_linear_scale, uniform_c0, uniform_c1, uniform_c2, uniform_c3, uniform_c4, uniform_beta;
 };
 
 }  // namespace movit
