@@ -18,7 +18,6 @@ float movit_texel_subpixel_precision;
 bool movit_srgb_textures_supported;
 bool movit_timer_queries_supported;
 int movit_num_wrongly_rounded;
-bool movit_shader_rounding_supported;
 MovitShaderModel movit_shader_model;
 
 // The rules for objects with nontrivial constructors in static scope
@@ -79,9 +78,11 @@ void measure_texel_subpixel_precision()
 
 	glViewport(0, 0, width, 1);
 
+	vector<string> frag_shader_outputs;
 	GLuint glsl_program_num = resource_pool.compile_glsl_program(
 		read_version_dependent_file("vs", "vert"),
-		read_version_dependent_file("texture1d", "frag"));
+		read_version_dependent_file("texture1d", "frag"),
+		frag_shader_outputs);
 	glUseProgram(glsl_program_num);
 	check_error();
 	glUniform1i(glGetUniformLocation(glsl_program_num, "tex"), 0);  // Bind the 2D sampler.
@@ -211,9 +212,11 @@ void measure_roundoff_problems()
 
 	glViewport(0, 0, 512, 1);
 
+	vector<string> frag_shader_outputs;
 	GLuint glsl_program_num = resource_pool.compile_glsl_program(
 		read_version_dependent_file("vs", "vert"),
-		read_version_dependent_file("texture1d", "frag"));
+		read_version_dependent_file("texture1d", "frag"),
+		frag_shader_outputs);
 	glUseProgram(glsl_program_num);
 	check_error();
 	glUniform1i(glGetUniformLocation(glsl_program_num, "tex"), 0);  // Bind the 2D sampler.
@@ -314,7 +317,6 @@ bool check_extensions()
 	if (!epoxy_is_desktop_gl()) {
 		if (epoxy_gl_version() >= 30) {
 			movit_srgb_textures_supported = true;
-			movit_shader_rounding_supported = true;
 			return true;
 		} else {
 			fprintf(stderr, "Movit system requirements: GLES version %.1f is too old (GLES 3.0 needed).\n",
@@ -349,13 +351,6 @@ bool check_extensions()
 	movit_srgb_textures_supported =
 		(epoxy_gl_version() >= 21 || epoxy_has_gl_extension("GL_EXT_texture_sRGB"));
 
-	// We may want to use round() at the end of the final shader,
-	// if supported. We need either GLSL 1.30 or this extension to do that,
-	// and 1.30 brings with it other things that we don't want to demand
-	// for now.
-	movit_shader_rounding_supported =
-		(epoxy_gl_version() >= 30 || epoxy_has_gl_extension("GL_EXT_gpu_shader4"));
-
 	// The user can specify that they want a timing report for each
 	// phase in an effect chain. However, that depends on this extension;
 	// without it, we do cannot even create the query objects.
@@ -428,11 +423,22 @@ bool init_movit(const string& data_directory, MovitDebugLevel debug_level)
 	}
 
 	// Find out what shader model we should compile for.
+	// We need at least 1.30, due to use of (among others) integers.
 	if (epoxy_is_desktop_gl()) {
-		if (get_glsl_version() >= 1.30) {
+		if (get_glsl_version() < 1.30f) {
+			fprintf(stderr, "Movit system requirements: Needs at least GLSL version 1.30 (has version %.1f)\n",
+				get_glsl_version());
+			return false;
+		}
+		if (get_glsl_version() < 1.50f) {
 			movit_shader_model = MOVIT_GLSL_130;
 		} else {
-			movit_shader_model = MOVIT_GLSL_110;
+			// Note: All of our 1.50 shaders are identical to our 1.30 shaders,
+			// but OS X does not support 1.30; only 1.10 (which we don't support
+			// anymore) and 1.50 (and then only with core contexts). So we keep
+			// a second set of shaders around whose only difference is the different
+			// #version declaration.
+			movit_shader_model = MOVIT_GLSL_150;
 		}
 	} else {
 		movit_shader_model = MOVIT_ESSL_300;
