@@ -74,6 +74,9 @@ float lanczos_weight(float x)
 //
 // Solve for e = 1e-6 yields a step size of 0.0027, which to cover the range
 // 0..3 needs 1109 steps. We round up to the next power of two, just to be sure.
+//
+// You need to call lanczos_table_init_done before the first call to
+// lanczos_weight_cached.
 #define LANCZOS_TABLE_SIZE 2048
 bool lanczos_table_init_done = false;
 float lanczos_table[LANCZOS_TABLE_SIZE + 2];
@@ -88,11 +91,6 @@ void init_lanczos_table()
 
 float lanczos_weight_cached(float x)
 {
-	if (!lanczos_table_init_done) {
-		// Could in theory race between two threads if we are unlucky,
-		// but that is harmless, since they'll write the same data.
-		init_lanczos_table();
-	}
 	x = fabs(x);
 	if (x > LANCZOS_RADIUS) {
 		return 0.0f;
@@ -194,13 +192,13 @@ template<class T>
 void normalize_sum(Tap<T>* vals, unsigned num)
 {
 	for (int normalize_pass = 0; normalize_pass < 2; ++normalize_pass) {
-		double sum = 0.0;
+		float sum = 0.0;
 		for (unsigned i = 0; i < num; ++i) {
-			sum += to_fp64(vals[i].weight);
+			sum += to_fp32(vals[i].weight);
 		}
-		double inv_sum = 1.0 / sum;
+		float inv_sum = 1.0 / sum;
 		for (unsigned i = 0; i < num; ++i) {
-			vals[i].weight = from_fp64<T>(to_fp64(vals[i].weight) * inv_sum);
+			vals[i].weight = from_fp32<T>(to_fp32(vals[i].weight) * inv_sum);
 		}
 	}
 }
@@ -257,8 +255,8 @@ double compute_sum_sq_error(const Tap<float>* weights, unsigned num_weights,
 	// Find the effective range of the bilinear-optimized kernel.
 	// Due to rounding of the positions, this is not necessarily the same
 	// as the intended range (ie., the range of the original weights).
-	int lower_pos = int(floor(to_fp64(bilinear_weights[0].pos) * size - 0.5));
-	int upper_pos = int(ceil(to_fp64(bilinear_weights[num_bilinear_weights - 1].pos) * size - 0.5)) + 2;
+	int lower_pos = int(floor(to_fp32(bilinear_weights[0].pos) * size - 0.5));
+	int upper_pos = int(ceil(to_fp32(bilinear_weights[num_bilinear_weights - 1].pos) * size - 0.5)) + 2;
 	lower_pos = min<int>(lower_pos, lrintf(weights[0].pos * size - 0.5));
 	upper_pos = max<int>(upper_pos, lrintf(weights[num_weights - 1].pos * size - 0.5) + 1);
 
@@ -269,7 +267,7 @@ double compute_sum_sq_error(const Tap<float>* weights, unsigned num_weights,
 
 	// Now find the effective weights that result from this sampling.
 	for (unsigned i = 0; i < num_bilinear_weights; ++i) {
-		const float pixel_pos = to_fp64(bilinear_weights[i].pos) * size - 0.5f;
+		const float pixel_pos = to_fp32(bilinear_weights[i].pos) * size - 0.5f;
 		const int x0 = int(floor(pixel_pos)) - lower_pos;
 		const int x1 = x0 + 1;
 		const float f = lrintf((pixel_pos - (x0 + lower_pos)) / movit_texel_subpixel_precision) * movit_texel_subpixel_precision;
@@ -279,8 +277,8 @@ double compute_sum_sq_error(const Tap<float>* weights, unsigned num_weights,
 		assert(x0 < upper_pos - lower_pos);
 		assert(x1 < upper_pos - lower_pos);
 
-		effective_weights[x0] += to_fp64(bilinear_weights[i].weight) * (1.0 - f);
-		effective_weights[x1] += to_fp64(bilinear_weights[i].weight) * f;
+		effective_weights[x0] += to_fp32(bilinear_weights[i].weight) * (1.0 - f);
+		effective_weights[x1] += to_fp32(bilinear_weights[i].weight) * f;
 	}
 
 	// Subtract the desired weights to get the error.
@@ -453,7 +451,7 @@ SingleResamplePassEffect::SingleResamplePassEffect(ResampleEffect *parent)
 	register_float("offset", &offset);
 	register_float("zoom", &zoom);
 	register_uniform_sampler2d("sample_tex", &uniform_sample_tex);
-	register_uniform_int("num_samples", &uniform_num_samples);  // FIXME: What about GLSL pre-1.30?
+	register_uniform_int("num_samples", &uniform_num_samples);
 	register_uniform_float("num_loops", &uniform_num_loops);
 	register_uniform_float("slice_height", &uniform_slice_height);
 	register_uniform_float("sample_x_scale", &uniform_sample_x_scale);
@@ -461,6 +459,12 @@ SingleResamplePassEffect::SingleResamplePassEffect(ResampleEffect *parent)
 	register_uniform_float("whole_pixel_offset", &uniform_whole_pixel_offset);
 
 	glGenTextures(1, &texnum);
+
+	if (!lanczos_table_init_done) {
+		// Could in theory race between two threads if we are unlucky,
+		// but that is harmless, since they'll write the same data.
+		init_lanczos_table();
+	}
 }
 
 SingleResamplePassEffect::~SingleResamplePassEffect()
@@ -713,10 +717,12 @@ void SingleResamplePassEffect::set_gl_state(GLuint glsl_program_num, const strin
 	// We specifically do not want mipmaps on the input texture;
 	// they break minification.
 	Node *self = chain->find_node_for_effect(this);
-	glActiveTexture(chain->get_input_sampler(self, 0));
-	check_error();
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-	check_error();
+	if (chain->has_input_sampler(self, 0)) {
+		glActiveTexture(chain->get_input_sampler(self, 0));
+		check_error();
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+		check_error();
+	}
 }
 
 }  // namespace movit
