@@ -19,6 +19,7 @@
 #include <epoxy/wgl.h>
 #else
 #include <epoxy/glx.h>
+#include <epoxy/egl.h>
 #endif
 
 using namespace std;
@@ -126,10 +127,10 @@ string read_file(const string &filename)
 
 string read_version_dependent_file(const string &base, const string &extension)
 {
-	if (movit_shader_model == MOVIT_GLSL_110) {
-		return read_file(base + "." + extension);
-	} else if (movit_shader_model == MOVIT_GLSL_130) {
+	if (movit_shader_model == MOVIT_GLSL_130) {
 		return read_file(base + ".130." + extension);
+	} else if (movit_shader_model == MOVIT_GLSL_150) {
+		return read_file(base + ".150." + extension);
 	} else if (movit_shader_model == MOVIT_ESSL_300) {
 		return read_file(base + ".300es." + extension);
 	} else {
@@ -232,8 +233,8 @@ void combine_two_samples(float w1, float w2, float pos1, float pos2, float num_s
 	}
 
 	// Round to the desired precision. Note that this might take z outside the 0..1 range.
-	*offset = from_fp64<DestFloat>(pos1 + z * (pos2 - pos1));
-	z = (to_fp64(*offset) - pos1) / (pos2 - pos1);
+	*offset = from_fp32<DestFloat>(pos1 + z * (pos2 - pos1));
+	z = (to_fp32(*offset) - pos1) / (pos2 - pos1);
 
 	// Round to the minimum number of bits we have measured earlier.
 	// The card will do this for us anyway, but if we know what the real z
@@ -253,11 +254,11 @@ void combine_two_samples(float w1, float w2, float pos1, float pos2, float num_s
 	//   w = (a(1-z) + bz) / ((1-z)² + z²)
 	//
 	// If z had infinite precision, this would simply reduce to w = w1 + w2.
-	*total_weight = from_fp64<DestFloat>((w1 + z * (w2 - w1)) / (z * z + (1 - z) * (1 - z)));
+	*total_weight = from_fp32<DestFloat>((w1 + z * (w2 - w1)) / (z * z + (1 - z) * (1 - z)));
 
 	if (sum_sq_error != NULL) {
-		float err1 = to_fp64(*total_weight) * (1 - z) - w1;
-		float err2 = to_fp64(*total_weight) * z - w2;
+		float err1 = to_fp32(*total_weight) * (1 - z) - w1;
+		float err2 = to_fp32(*total_weight) * z - w2;
 		*sum_sq_error = err1 * err1 + err2 * err2;
 	}
 }
@@ -271,6 +272,21 @@ template
 void combine_two_samples<fp16_int_t>(float w1, float w2, float pos1, float pos2, float num_subtexels, float inv_num_subtexels,
                                      fp16_int_t *offset, fp16_int_t *total_weight, float *sum_sq_error);
 
+GLuint generate_vbo(GLint size, GLenum type, GLsizeiptr data_size, const GLvoid *data)
+{
+	GLuint vbo;
+	glGenBuffers(1, &vbo);
+	check_error();
+	glBindBuffer(GL_ARRAY_BUFFER, vbo);
+	check_error();
+	glBufferData(GL_ARRAY_BUFFER, data_size, data, GL_STATIC_DRAW);
+	check_error();
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	check_error();
+
+	return vbo;
+}
+
 GLuint fill_vertex_attribute(GLuint glsl_program_num, const string &attribute_name, GLint size, GLenum type, GLsizeiptr data_size, const GLvoid *data)
 {
 	int attrib = glGetAttribLocation(glsl_program_num, attribute_name.c_str());
@@ -278,13 +294,10 @@ GLuint fill_vertex_attribute(GLuint glsl_program_num, const string &attribute_na
 		return -1;
 	}
 
-	GLuint vbo;
-	glGenBuffers(1, &vbo);
-	check_error();
+	GLuint vbo = generate_vbo(size, type, data_size, data);
+
 	glBindBuffer(GL_ARRAY_BUFFER, vbo);
 	check_error();
-	glBufferData(GL_ARRAY_BUFFER, data_size, data, GL_STATIC_DRAW);
-	check_error();
 	glEnableVertexAttribArray(attrib);
 	check_error();
 	glVertexAttribPointer(attrib, size, type, GL_FALSE, 0, BUFFER_OFFSET(0));
@@ -333,6 +346,10 @@ void *get_gl_context_identifier()
 #elif defined(WIN32)
 	return (void *)wglGetCurrentContext();
 #else
+	void *ret = (void *)eglGetCurrentContext();
+	if (ret != NULL) {
+		return ret;
+	}
 	return (void *)glXGetCurrentContext();
 #endif
 }
