@@ -1,12 +1,16 @@
-// Unit tests for YCbCrInput.
+// Unit tests for YCbCrInput. Also tests the matrix functions in ycbcr.cpp directly.
 
 #include <epoxy/gl.h>
 #include <stddef.h>
 
+#include <Eigen/Core>
+#include <Eigen/LU>
+
 #include "effect_chain.h"
 #include "gtest/gtest.h"
 #include "test_util.h"
 #include "util.h"
+#include "resource_pool.h"
 #include "ycbcr_input.h"
 
 namespace movit {
@@ -351,7 +355,7 @@ TEST(YCbCrInputTest, Subsampling420WithNonCenteredSamples) {
 
 	// Y'CbCr isn't 100% accurate (the input values are rounded),
 	// so we need some leeway.
-	expect_equal(expected_data, out_data, width, height, 0.01, 0.001);
+	expect_equal(expected_data, out_data, width, height, 0.01, 0.0012);
 }
 
 // Yes, some 4:2:2 formats actually have this craziness.
@@ -538,4 +542,150 @@ TEST(YCbCrInputTest, CombinedCbAndCr) {
 	expect_equal(expected_data, out_data, 4 * width, height, 0.025, 0.002);
 }
 
+TEST(YCbCrInputTest, ExternalTexture) {
+	const int width = 1;
+	const int height = 5;
+
+	// Pure-color test inputs, calculated with the formulas in Rec. 601
+	// section 2.5.4.
+	unsigned char y[width * height] = {
+		16, 235, 81, 145, 41,
+	};
+	unsigned char cb[width * height] = {
+		128, 128, 90, 54, 240,
+	};
+	unsigned char cr[width * height] = {
+		128, 128, 240, 34, 110,
+	};
+	float expected_data[4 * width * height] = {
+		0.0, 0.0, 0.0, 1.0,
+		1.0, 1.0, 1.0, 1.0,
+		1.0, 0.0, 0.0, 1.0,
+		0.0, 1.0, 0.0, 1.0,
+		0.0, 0.0, 1.0, 1.0,
+	};
+	float out_data[4 * width * height];
+
+	EffectChainTester tester(NULL, width, height);
+
+	ImageFormat format;
+	format.color_space = COLORSPACE_sRGB;
+	format.gamma_curve = GAMMA_sRGB;
+
+	YCbCrFormat ycbcr_format;
+	ycbcr_format.luma_coefficients = YCBCR_REC_601;
+	ycbcr_format.full_range = false;
+	ycbcr_format.num_levels = 256;
+	ycbcr_format.chroma_subsampling_x = 1;
+	ycbcr_format.chroma_subsampling_y = 1;
+	ycbcr_format.cb_x_position = 0.5f;
+	ycbcr_format.cb_y_position = 0.5f;
+	ycbcr_format.cr_x_position = 0.5f;
+	ycbcr_format.cr_y_position = 0.5f;
+
+	// Make a texture for the Cb data; keep the others as regular uploads.
+	ResourcePool pool;
+	GLuint cb_tex = pool.create_2d_texture(GL_R8, width, height);
+	check_error();
+	glBindTexture(GL_TEXTURE_2D, cb_tex);
+	check_error();
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	check_error();
+	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+	check_error();
+	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, width, height, GL_RED, GL_UNSIGNED_BYTE, cb);
+	check_error();
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+	check_error();
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+	check_error();
+
+	YCbCrInput *input = new YCbCrInput(format, ycbcr_format, width, height);
+	input->set_pixel_data(0, y);
+	input->set_texture_num(1, cb_tex);
+	input->set_pixel_data(2, cr);
+	tester.get_chain()->add_input(input);
+
+	tester.run(out_data, GL_RGBA, COLORSPACE_sRGB, GAMMA_sRGB);
+
+	pool.release_2d_texture(cb_tex);
+
+	// Y'CbCr isn't 100% accurate (the input values are rounded),
+	// so we need some leeway.
+	expect_equal(expected_data, out_data, 4 * width, height, 0.025, 0.002);
+}
+
+TEST(YCbCrTest, WikipediaRec601ForwardMatrix) {
+	YCbCrFormat ycbcr_format;
+	ycbcr_format.luma_coefficients = YCBCR_REC_601;
+	ycbcr_format.full_range = false;
+	ycbcr_format.num_levels = 256;
+
+	float offset[3];
+	Eigen::Matrix3d ycbcr_to_rgb;
+	compute_ycbcr_matrix(ycbcr_format, offset, &ycbcr_to_rgb);
+
+	Eigen::Matrix3d rgb_to_ycbcr = ycbcr_to_rgb.inverse() * 255.0;
+
+	// Values from https://en.wikipedia.org/wiki/YCbCr#ITU-R_BT.601_conversion.
+	EXPECT_NEAR(  65.481, rgb_to_ycbcr(0,0), 1e-3);
+	EXPECT_NEAR( 128.553, rgb_to_ycbcr(0,1), 1e-3);
+	EXPECT_NEAR(  24.966, rgb_to_ycbcr(0,2), 1e-3);
+
+	EXPECT_NEAR( -37.797, rgb_to_ycbcr(1,0), 1e-3);
+	EXPECT_NEAR( -74.203, rgb_to_ycbcr(1,1), 1e-3);
+	EXPECT_NEAR( 112.000, rgb_to_ycbcr(1,2), 1e-3);
+
+	EXPECT_NEAR( 112.000, rgb_to_ycbcr(2,0), 1e-3);
+	EXPECT_NEAR( -93.786, rgb_to_ycbcr(2,1), 1e-3);
+	EXPECT_NEAR( -18.214, rgb_to_ycbcr(2,2), 1e-3);
+
+	EXPECT_NEAR( 16.0, offset[0] * 255.0, 1e-3);
+	EXPECT_NEAR(128.0, offset[1] * 255.0, 1e-3);
+	EXPECT_NEAR(128.0, offset[2] * 255.0, 1e-3);
+}
+
+TEST(YCbCrTest, WikipediaJPEGMatrices) {
+	YCbCrFormat ycbcr_format;
+	ycbcr_format.luma_coefficients = YCBCR_REC_601;
+	ycbcr_format.full_range = true;
+	ycbcr_format.num_levels = 256;
+
+	float offset[3];
+	Eigen::Matrix3d ycbcr_to_rgb;
+	compute_ycbcr_matrix(ycbcr_format, offset, &ycbcr_to_rgb);
+
+	// Values from https://en.wikipedia.org/wiki/YCbCr#JPEG_conversion.
+	EXPECT_NEAR( 1.00000, ycbcr_to_rgb(0,0), 1e-5);
+	EXPECT_NEAR( 0.00000, ycbcr_to_rgb(0,1), 1e-5);
+	EXPECT_NEAR( 1.40200, ycbcr_to_rgb(0,2), 1e-5);
+
+	EXPECT_NEAR( 1.00000, ycbcr_to_rgb(1,0), 1e-5);
+	EXPECT_NEAR(-0.34414, ycbcr_to_rgb(1,1), 1e-5);
+	EXPECT_NEAR(-0.71414, ycbcr_to_rgb(1,2), 1e-5);
+
+	EXPECT_NEAR( 1.00000, ycbcr_to_rgb(2,0), 1e-5);
+	EXPECT_NEAR( 1.77200, ycbcr_to_rgb(2,1), 1e-5);
+	EXPECT_NEAR( 0.00000, ycbcr_to_rgb(2,2), 1e-5);
+
+	Eigen::Matrix3d rgb_to_ycbcr = ycbcr_to_rgb.inverse();
+
+	// Same.
+	EXPECT_NEAR( 0.299000, rgb_to_ycbcr(0,0), 1e-6);
+	EXPECT_NEAR( 0.587000, rgb_to_ycbcr(0,1), 1e-6);
+	EXPECT_NEAR( 0.114000, rgb_to_ycbcr(0,2), 1e-6);
+
+	EXPECT_NEAR(-0.168736, rgb_to_ycbcr(1,0), 1e-6);
+	EXPECT_NEAR(-0.331264, rgb_to_ycbcr(1,1), 1e-6);
+	EXPECT_NEAR( 0.500000, rgb_to_ycbcr(1,2), 1e-6);
+
+	EXPECT_NEAR( 0.500000, rgb_to_ycbcr(2,0), 1e-6);
+	EXPECT_NEAR(-0.418688, rgb_to_ycbcr(2,1), 1e-6);
+	EXPECT_NEAR(-0.081312, rgb_to_ycbcr(2,2), 1e-6);
+
+	EXPECT_NEAR(  0.0, offset[0] * 255.0, 1e-3);
+	EXPECT_NEAR(128.0, offset[1] * 255.0, 1e-3);
+	EXPECT_NEAR(128.0, offset[2] * 255.0, 1e-3);
+}
+
 }  // namespace movit
